import sysactions


def get_store_id():
    # type: (...) -> str

    return sysactions.get_storewide_config("Store.Id")


def get_boh_url():
    # type: (...) -> str

    return sysactions.get_storewide_config("BackOffice.ServerURL")


def get_boh_api_key():
    # type: (...) -> str

    return sysactions.get_storewide_config("BackOffice.ApiKey")
