from functions_authorization import get_function_authorization_level_from_config
from get_keyboard_comments_config import get_keyboard_comments_config
from get_min_value_to_ask_bordereau_justify_config import get_min_value_to_ask_bordereau_justify_config
from show_numpad_for_customer_name import show_numpad_for_customer_name
