from typing import List


class OperatorGoals(object):
    def __init__(
        self,
        operator_id,        # type: str
        amount_goal,        # type: float
        amount_sold,        # type: float
        average_ticket,     # type: float
        item_goals,         # type: List
        tickets,            # type: float
    ):
        # type (...) -> None

        self.id = operator_id
        self.amount_goal = amount_goal
        self.amount_sold = amount_sold
        self.average_ticket = average_ticket
        self.item_goals = item_goals
        self.tickets = tickets
