# encoding: utf-8
from _JustificationDiscountBodyDto import JustificationDiscountBodyDto
from _JustificationDiscountHeaderDto import JustificationDiscountHeaderDto


class JustificationDiscountGeneratorDto(object):
    def __init__(
        self,
        header,      # type: JustificationDiscountHeaderDto
        body,        # type: JustificationDiscountBodyDto
    ):
        # type: (...) -> None

        self.header = header
        self.body = body
