from reportsdata.samplereport import SampleReportGenerator, SampleReportPresenter


class ReportGenerator(object):
    @staticmethod
    def generate(mb_context, pos_id, param1, param2, param3, param4):
        return SampleReportGenerator(), SampleReportPresenter()
