class GetUsedServiceRequest(object):
    def __init__(
        self,
        pos_id,         # type: str
        device_type,    # type: str
        sector,         # type: str
    ):
        # type: (...) -> None

        self.pos_id = pos_id
        self.device_type = device_type
        self.sector = sector
