# encoding: utf-8
from typing import Dict


class TefCancellationBodyDto(object):
    def __init__(
        self,
        tenders_by_date,         # type: Dict
        card_brands     # type: Dict
    ):
        # type: (...) -> None

        self.tenders_by_date = tenders_by_date
        self.card_brands = card_brands
